/* qmmp3info.cpp
 *
 * $Id: qmmp3info.cpp,v 1.9 2002/05/22 00:41:19 kyllingstad Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "qmmp3info.h"
#include "util.h"

# include "libid3tag/id3tag.h"

#include <iostream.h>
#include <stdlib.h>
#include <cassert>
#include <limits.h>

/*!
  \class QmMp3Info qmmp3info.h
  \brief Provides functionality to retrieve ID3 fields.
*/


/*!
  Creates an MP3 information object to retrieve information from \a filename.
*/

QmMp3Info::QmMp3Info(
	const QString &filename)
	: m_Filename(filename)
{
	for (int i=0; i<QmSongInfo::Fields; ++i)
		m_Fields[i] = blank();

	readLibid3tag();

	// rk: the heuristics could stand improvement:
	if (filename.contains("various", false))
		m_Fields[MultiArtist] = "1";

	if (m_Fields[Length].isEmpty())
	{
		int len = getMp3LengthInSeconds(filename);
		if (len != -1)
			m_Fields[Length] = QString::number(len);
	}
}


/*!

*/

QmMp3Info::~QmMp3Info()
{
}


/*!
  \return The value of the field \a f, or blank if the field does not exist.
*/

const QString&
QmMp3Info::get(
	Field f) const
{
	return m_Fields[f];
}


/*!
  Read id3 tags using libid3tag - copied from madlib
*/

void
QmMp3Info::readLibid3tag()
{
    id3_file *file = id3_file_open(m_Filename.latin1(), ID3_FILE_MODE_READONLY);
    if (file == 0)
		return;

	id3_tag *tag = id3_file_tag(file);

	struct {
		char const *id;
		Field field;
	} const info[] = {
		{ ID3_FRAME_TITLE,  Title  },
		{ ID3_FRAME_ARTIST, Artist },
		{ ID3_FRAME_ALBUM,  Album  },
		{ ID3_FRAME_YEAR,   Year   },
		{ ID3_FRAME_TRACK,  Track  },
		{ ID3_FRAME_GENRE,  Genre  },
		{ "TLEN",           Length }, 
	};

	/* text information */

	id3_frame *frame;
	unsigned int i;
	const id3_ucs4_t *ucs4;
	char *latin1;
	for (i = 0; i < sizeof(info) / sizeof(info[0]); ++i) {
		union id3_field *field;
		unsigned int nstrings;

		frame = id3_tag_findframe(tag, info[i].id, 0);
		if (frame == 0)
			continue;

		field    = &frame->fields[1];
		nstrings = id3_field_getnstrings(field);

		for (unsigned int j = 0; j < nstrings; ++j)
		{
			ucs4 = id3_field_getstrings(field, j);
			assert(ucs4);

			latin1 = reinterpret_cast<char *>(id3_ucs4_latin1duplicate(ucs4));
			if (latin1 == 0)
				goto fail;

			if (strcmp(info[i].id, ID3_FRAME_GENRE) == 0)
				readGenre(latin1);
			else if (strcmp(info[i].id, "TLEN") == 0)
			{
				bool ok;
 				long len = QString(latin1).toLong(&ok) / 1000; 
				
 				if(ok)
 					m_Fields[QmSongInfo::Length] =  QString::number(len);
			}
			else if (j == 0)
				m_Fields[info[i].field] = latin1;
			else
				m_Fields[info[i].field] += latin1; 
			
			delete(latin1);
		}
	}

	/* comments */

	for (i=0; true; ++i)
	{
		frame = id3_tag_findframe(tag, ID3_FRAME_COMMENT, i);
		if (frame == 0)
			break;

		ucs4 = id3_field_getstring(&frame->fields[2]);
		assert(ucs4);

		if (*ucs4)
			continue;

		ucs4 = id3_field_getfullstring(&frame->fields[3]);
		assert(ucs4);

		latin1 = reinterpret_cast<char *>(id3_ucs4_latin1duplicate(ucs4));
		if (latin1 == 0)
			goto fail;

		m_Fields[Comment] = latin1;
		delete(latin1);
	}

	if (0) {
	fail: 
		std::cerr << "not enough memory to display tag";
	}
	id3_file_close(file);
}


/*!
  read genre into m_Fields[Genre] from a id3 genre field
*/

void
QmMp3Info::readGenre(const char *from)
{
	if (*from == '(')
	{
		char* end;
		long genre = strtol(from+1, &end, 10);
		if (genre==LONG_MIN || genre==LONG_MAX || end==(from+1))
			return;
		if (genre>=0 && genre<126)
			m_Fields[Genre] == genres[genre];
		/*! \todo read refinement */
	}
	else
		m_Fields[Genre] = QString(from);
}


// list from: http://www.id3.org/id3v2.3.0.html
const char*
QmMp3Info::genres[] =
{"Blues",
 "Classic Rock",
 "Country",
 "Dance",
 "Disco",
 "Funk",
 "Grunge",
 "Hip-Hop",
 "Jazz",
 "Metal",
 "New Age",
 "Oldies",
 "Other",
 "Pop",
 "R&B",
 "Rap",
 "Reggae",
 "Rock",
 "Techno",
 "Industrial",
 "Alternative",
 "Ska",
 "Death Metal",
 "Pranks",
 "Soundtrack",
 "Euro-Techno",
 "Ambient",
 "Trip-Hop",
 "Vocal",
 "Jazz+Funk",
 "Fusion",
 "Trance",
 "Classical",
 "Instrumental",
 "Acid",
 "House",
 "Game",
 "Sound Clip",
 "Gospel",
 "Noise",
 "AlternRock",
 "Bass",
 "Soul",
 "Punk",
 "Space",
 "Meditative",
 "Instrumental Pop",
 "Instrumental Rock",
 "Ethnic",
 "Gothic",
 "Darkwave",
 "Techno-Industrial",
 "Electronic",
 "Pop-Folk",
 "Eurodance",
 "Dream",
 "Southern Rock",
 "Comedy",
 "Cult",
 "Gangsta",
 "Top 40",
 "Christian Rap",
 "Pop/Funk",
 "Jungle",
 "Native American",
 "Cabaret",
 "New Wave",
 "Psychadelic",
 "Rave",
 "Showtunes",
 "Trailer",
 "Lo-Fi",
 "Tribal",
 "Acid Punk",
 "Acid Jazz",
 "Polka",
 "Retro",
 "Musical",
 "Rock & Roll",
 "Hard Rock",
 "Folk",
 "Folk-Rock",
 "National Folk",
 "Swing",
 "Fast Fusion",
 "Bebob",
 "Latin",
 "Revival",
 "Celtic",
 "Bluegrass",
 "Avantgarde",
 "Gothic Rock",
 "Progressive Rock",
 "Psychedelic Rock",
 "Symphonic Rock",
 "Slow Rock",
 "Big Band",
 "Chorus",
 "Easy Listening",
 "Acoustic",
 "Humour",
 "Speech",
 "Chanson",
 "Opera",
 "Chamber Music",
 "Sonata",
 "Symphony",
 "Booty Bass",
 "Primus",
 "Porn Groove",
 "Satire",
 "Slow Jam",
 "Club",
 "Tango",
 "Samba",
 "Folklore",
 "Ballad",
 "Power Ballad",
 "Rhythmic Soul",
 "Freestyle",
 "Duet",
 "Punk Rock",
 "Drum Solo",
 "A capella",
 "Euro-House",
 "Dance Hall"};
