/* qmrecentfiles.cpp
 *
 * $Id: qmrecentfiles.cpp,v 1.1 2002/05/22 00:41:19 kyllingstad Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

/*!
  \class QmRecentFiles
  \brief A PopupMenu with the most recently loaded playlists, playlist trees and directories

  The popup menu is added to the "File" menu in qmcontrolbar, and is maintained from the mainwindow
  playlistAdd* functions.

  Uses qmconfig "stuff" "recent-files-max-{playlists,playlisttrees,directories}" to get number of
  entries to store (doesn't have gui config).
*/


#include "qmrecentfiles.h"
#include "qmcontrolbar.h"

#include <qfileinfo.h>

QmAutoPtr<QmRecentFiles> QmRecentFiles::s_pInstance;


QmRecentFiles::QmRecentFiles()
{
	m_pMenu = new QPopupMenu();
	m_pDirectoryMenu = new QPopupMenu();
}


QmRecentFiles::~QmRecentFiles()
{
	delete m_pDirectoryMenu;
	delete m_pMenu;
}


/*!
  Add a playlist to the recent files list
*/
  
void
QmRecentFiles::addPlayList(const QString &playlist)
{
	add(playlist, &m_PlayLists, static_cast<unsigned int>(QmConfig::instance()->getInt("stuff", "recent-files-max-playlists")));
}


/*!
  Add a playlisttree to the recent files list
*/

void
QmRecentFiles::addPlayListTree(const QString &tree)
{
	add(tree, &m_PlayListTrees, static_cast<unsigned int>(QmConfig::instance()->getInt("stuff", "recent-files-max-playlisttrees")));
}


/*!
  Add a directory to the recent files list
*/

void
QmRecentFiles::addDirectory(const QString &dir)
{
	add(dir, &m_Directories, static_cast<unsigned int>(QmConfig::instance()->getInt("stuff", "recent-files-max-directories")));
}


/*!
  Refreshes menu after change, called from add() and load().
 */

void
QmRecentFiles::updateMenu()
{
	m_pMenu->clear();
	m_pDirectoryMenu->clear();
	unsigned int menuIndex = 0;
	QFileInfo fi;
	QString tmp;
	if (!m_Directories.isEmpty())
	{
		m_pMenu->insertItem( "Directories", m_pDirectoryMenu);
		for (QStringList::Iterator it = m_Directories.begin(); it != m_Directories.end(); )
		{
			fi.setFile(*it);
			if (!fi.exists())
				it = m_Directories.remove(it);
			else
				m_pDirectoryMenu->setItemParameter(m_pDirectoryMenu->insertItem(QString("%1 (%2)").arg(fi.baseName()).arg(*it++), this, SLOT(clicked(int))), menuIndex++);
		}
	}
	for (QStringList::Iterator it = m_PlayLists.begin(); it != m_PlayLists.end(); )
	{
		fi.setFile(*it);
		if (!fi.exists()) 
			it = m_PlayLists.remove(it);
		else
			m_pMenu->setItemParameter(m_pMenu->insertItem(QString("%1 (%2)").arg(fi.baseName()).arg(*it++), this, SLOT(clicked(int))), menuIndex++);
	}
	for (QStringList::Iterator it = m_PlayListTrees.begin(); it != m_PlayListTrees.end(); )
	{
		fi.setFile(*it);
		if (!fi.exists()) 
			it = m_PlayListTrees.remove(it);
		else
			m_pMenu->setItemParameter(m_pMenu->insertItem(QString("%1 (%2)").arg(fi.baseName()).arg(*it++), this, SLOT(clicked(int))), menuIndex++);
	}
}


/*!
  Save list in QmConfig
*/

void
QmRecentFiles::save()
{
	QmConfig *conf = QmConfig::instance();

	int i=0;
	for (QStringList::ConstIterator it = m_PlayLists.begin(); it != m_PlayLists.end(); ++i, ++it)
		conf->set("recent-files", QString("playlist-%1").arg(i), *it);

	i=0;
	for (QStringList::ConstIterator it = m_PlayListTrees.begin(); it != m_PlayListTrees.end(); ++i, ++it)
		conf->set("recent-files", QString("playlisttree-%1").arg(i), *it);

	i=0;
	for (QStringList::ConstIterator it = m_Directories.begin(); it != m_Directories.end(); ++i, ++it)
		conf->set("recent-files", QString("directory-%1").arg(i), *it);
	
	// remove any old entries
	i = m_PlayLists.count();
	while (conf->remove("recent-files", QString("playlist-%1").arg(i)))
		++i;
	i = m_PlayListTrees.count();
	while (conf->remove("recent-files", QString("playlisttree-%1").arg(i)))
		++i;
	i = m_Directories.count();
	while (conf->remove("recent-files", QString("directory-%1").arg(i)))
		++i;
}


/*!
  Load list from QmConfig
*/

void
QmRecentFiles::load()
{
	QmConfig *conf = QmConfig::instance();

	QString tmp;
	for (int i=0; true; ++i)
	{
		tmp = conf->getString("recent-files", QString("playlist-%1").arg(i)); 
		if (tmp != QString::null)
			m_PlayLists.append(tmp);
		else
			break;
	}
		
	for (int i=0; true; ++i)
	{
		tmp = conf->getString("recent-files", QString("playlisttree-%1").arg(i)); 
		if (tmp != QString::null)
			m_PlayListTrees.append(tmp);
		else
			break;
	}

	for (int i=0; true; ++i)
	{
		tmp = conf->getString("recent-files", QString("directory-%1").arg(i)); 
		if (tmp != QString::null)
			m_Directories.append(tmp);
		else
			break;
	}

	updateMenu();
}


/*!
  Popup the menu 
*/

void
QmRecentFiles::popup()
{
	QPoint p = QmMainWindow::mainwin->controlBar()->mapToGlobal(QmMainWindow::mainwin->controlBar()->pos());
	p.setY(p.y() + QmMainWindow::mainwin->controlBar()->height());
	m_pMenu->popup(p);
}


/*!
  Menu callback
 */

void
QmRecentFiles::clicked(int menuIndex)
{
	if (static_cast<unsigned int>(menuIndex) < m_Directories.count())
	{
		QString add = m_Directories[menuIndex];
		m_Directories.remove(add);
		QmMainWindow::mainwin->playListAppendDir(add);
	}
	else if (static_cast<unsigned int>(menuIndex) < m_Directories.count() + m_PlayLists.count())
	{
		QString add = m_PlayLists[menuIndex-m_Directories.count()];
		m_PlayLists.remove(add);
		QmMainWindow::mainwin->playListAdd(add);
	}
	else
	{
		QString add = m_PlayListTrees[menuIndex-m_Directories.count()-m_PlayLists.count()];
		m_PlayListTrees.remove(add);
		QmMainWindow::mainwin->loadPlayListTree(add);
	}
}
			

/*!
  Add playlist/playlisttree/directory, for internal use.
 */

void
QmRecentFiles::add(const QString &file, QStringList *to, unsigned int max)
{
	if (to->contains(file) > 0)
		to->remove(file);
	to->prepend(file);

	while (to->count() > max)
		to->remove(to->fromLast());

	updateMenu();
}


/*!
  \return recent files singleton
*/

QmRecentFiles*
QmRecentFiles::instance()
{
	if (QmRecentFiles::s_pInstance.get() == 0)
		s_pInstance.reset(new QmRecentFiles());
	return reinterpret_cast<QmRecentFiles *>(QmRecentFiles::s_pInstance.get());
}
